from rest_framework.exceptions import ValidationError
from rest_framework.views import exception_handler
from rest_framework_simplejwt.exceptions import InvalidToken
from django.utils.translation import gettext_lazy as _


def custom_exception_handler(exc, context):
    response = exception_handler(exc, context)

    if response is not None:
        if isinstance(exc, ValidationError):
            response.data = {
                "error": _("please provide correct data."),
                "fields": response.data,
            }
        elif isinstance(exc, InvalidToken):
            response.data = {
                "error": _("please login again"),
            }
        else:
            response.data = {"error": str(exc)}
        response.data["status_code"] = response.status_code

    return response


import copy
from typing import Any, Dict, List, Optional

from django.db import models
from django.http import HttpRequest
from django.urls import reverse
from media_uploader_widget.widgets import MediaUploaderWidget
from tinymce.widgets import TinyMCE
from unfold.admin import ModelAdmin, UnfoldAction, UnfoldBooleanSwitchWidget


class BaseModelAdmin(ModelAdmin):
    formfield_overrides = {
        models.TextField: {
            "widget": TinyMCE(),
        },
        models.FileField: {
            "widget": MediaUploaderWidget,
        },
        models.ImageField: {
            "widget": MediaUploaderWidget,
        },
    }
    list_per_page = 30
    compressed_fields = True
    # Display submit button in filters
    list_filter_submit = True

    def get_actions_detail(
        self, request: HttpRequest, object_id: int = None
    ) -> List[UnfoldAction]:
        return self._filter_unfold_actions_by_permissions(
            request, self._get_base_actions_detail()
        )

    def changeform_view(
        self,
        request: HttpRequest,
        object_id: Optional[str] = None,
        form_url: str = "",
        extra_context: Optional[Dict[str, bool]] = None,
    ) -> Any:
        if extra_context is None:
            extra_context = {}

        new_formfield_overrides = copy.deepcopy(self.formfield_overrides)
        new_formfield_overrides.update(
            {models.BooleanField: {"widget": UnfoldBooleanSwitchWidget}}
        )

        self.formfield_overrides = new_formfield_overrides

        actions = []
        if object_id:
            for action in self.get_actions_detail(request, object_id):
                actions.append(
                    {
                        "title": action.description,
                        "attrs": action.method.attrs,
                        "path": reverse(
                            f"admin:{action.action_name}", args=(object_id,)
                        ),
                    }
                )

        extra_context.update(
            {
                "actions_submit_line": self.get_actions_submit_line(request, object_id),
                "actions_detail": actions,
            }
        )

        return super(ModelAdmin, self).changeform_view(
            request, object_id, form_url, extra_context
        )



"""
Basic building blocks for generic class based views.

We don't bind behaviour to http method handlers yet,
which allows mixin classes to be composed in interesting ways.
"""

from rest_framework import status
from rest_framework.settings import api_settings

from .response import CustomResponse
from django.utils.translation import gettext as _


class CustomCreateModelMixin:
    """
    Create a model instance.
    """

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return CustomResponse(
            data=serializer.data,
            status=status.HTTP_201_CREATED,
            headers=headers,
            message=f"{self.get_serializer().Meta.model._meta.verbose_name} "
            + _("created successfully"),
        )

    def perform_create(self, serializer):
        serializer.save()

    def get_success_headers(self, data):
        try:
            return {"Location": str(data[api_settings.URL_FIELD_NAME])}
        except (TypeError, KeyError):
            return {}


class CustomListModelMixin:
    """
    List a queryset.
    """

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return CustomResponse(
            data=serializer.data,
            status=status.HTTP_200_OK,
            message=f"{self.get_serializer().Meta.model._meta.verbose_name_plural} "
            + _("retrieved successfully"),
        )


class CustomRetrieveModelMixin:
    """
    Retrieve a model instance.
    """

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        return CustomResponse(
            serializer.data,
            status=status.HTTP_200_OK,
            message=f"{self.get_serializer().Meta.model._meta.verbose_name} "
            + _("retrieved successfully"),
        )


class CustomUpdateModelMixin:
    """
    Update a model instance.
    """

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return CustomResponse(
            data=serializer.data,
            message=f"{self.get_serializer().Meta.model._meta.verbose_name} "
            + _("updated successfully"),
            status=status.HTTP_200_OK,
        )

    def perform_update(self, serializer):
        serializer.save()

    def partial_update(self, request, *args, **kwargs):
        kwargs["partial"] = True
        return self.update(request, *args, **kwargs)


class CustomDestroyModelMixin:
    """
    Destroy a model instance.
    """

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)
        return CustomResponse(status=status.HTTP_204_NO_CONTENT)

    def perform_destroy(self, instance):
        instance.delete()



from django.db import models
from image_uploader_widget.widgets import ImageUploaderWidget
from tinymce.widgets import TinyMCE
from unfold.contrib.inlines.admin import (
    NonrelatedStackedInline,
    NonrelatedTabularInline,
    StackedInline,
    TabularInline,
)


class TabularInline(TabularInline):
    """
    Custom inline class for Django admin interface that organizes inline models
    into separate tabs based on specified class names.

    This class extends `admin.TabularInline` and adds functionality to dynamically
    assign inlines to specific tabs in the admin interface based on their class names.

    Usage:
        1. Define inline classes inheriting from `TabInline`.
        2. Specify these inline classes in the `inlines` attribute of your `ModelAdmin`.
        3. Use the `fieldsets` attribute to organize fields into tabs, using the `classes`
           attribute to specify which inline classes should appear in each tab.
    Example:
        class CategoryInline(TabInline):
            ...
        class MyModelAdmin(ModelAdmin):
            ...
            fieldsets = (
                ...,
                (
                    "Category Section",
                    {
                        "classes": ["tab", "CategoryInline" ],
                        "fields": [],
                    },
                ),
    """

    @property
    def class_name(self):
        return self.__class__.__name__

    def get_formset(self, request, obj=None, **kwargs):
        formset = super().get_formset(request, obj, **kwargs)
        formset.class_name = self.class_name
        return formset

    formfield_overrides = {
        models.TextField: {"widget": TinyMCE()},
        models.ImageField: {"widget": ImageUploaderWidget},
        models.FileField: {"widget": ImageUploaderWidget},
    }


class StackedInline(StackedInline):
    """
    Custom inline class for Django admin interface that organizes inline models
    into separate tabs based on specified class names.

    This class extends `admin.StackedInline` and adds functionality to dynamically
    assign inlines to specific tabs in the admin interface based on their class names.

    Usage:
        1. Define inline classes inheriting from `TabInline`.
        2. Specify these inline classes in the `inlines` attribute of your `ModelAdmin`.
        3. Use the `fieldsets` attribute to organize fields into tabs, using the `classes`
           attribute to specify which inline classes should appear in each tab.
    Example:
        class CategoryInline(TabInline):
            ...
        class MyModelAdmin(ModelAdmin):
            ...
            fieldsets = (
                ...,
                (
                    "Category Section",
                    {
                        "classes": ["tab", "CategoryInline" ],
                        "fields": [],
                    },
                ),
    """

    @property
    def class_name(self):
        return self.__class__.__name__

    def get_formset(self, request, obj: None = ..., **kwargs):
        formset = super().get_formset(request, obj, **kwargs)
        formset.class_name = self.class_name
        return formset

    formfield_overrides = {
        models.TextField: {"widget": TinyMCE()},
        models.ImageField: {"widget": ImageUploaderWidget},
        models.FileField: {"widget": ImageUploaderWidget},
    }


class NonrelatedStackedInline(NonrelatedStackedInline):
    """
    Custom inline class for Django admin interface that organizes inline models
    into separate tabs based on specified class names.

    This class extends `admin.StackedInline` and adds functionality to dynamically
    assign inlines to specific tabs in the admin interface based on their class names.

    Usage:
        1. Define inline classes inheriting from `TabInline`.
        2. Specify these inline classes in the `inlines` attribute of your `ModelAdmin`.
        3. Use the `fieldsets` attribute to organize fields into tabs, using the `classes`
           attribute to specify which inline classes should appear in each tab.
    Example:
        class CategoryInline(TabInline):
            ...
        class MyModelAdmin(ModelAdmin):
            ...
            fieldsets = (
                ...,
                (
                    "Category Section",
                    {
                        "classes": ["tab", "CategoryInline" ],
                        "fields": [],
                    },
                ),
    """

    @property
    def class_name(self):
        return self.__class__.__name__

    def get_form_queryset(self, obj):
        """
        Gets all nonrelated objects needed for inlines. Method must be implemented.
        """
        return self.model.objects.all()

    def save_new_instance(self, parent, instance):
        """
        Extra save method which can for example update inline instances based on current
        main model object. Method must be implemented.
        """
        pass

    formfield_overrides = {
        models.TextField: {"widget": TinyMCE()},
        models.ImageField: {"widget": ImageUploaderWidget},
        models.FileField: {"widget": ImageUploaderWidget},
    }


class NonrelatedTabularInline(NonrelatedTabularInline):
    """
    Custom inline class for Django admin interface that organizes inline models
    into separate tabs based on specified class names.

    This class extends `admin.TabularInline` and adds functionality to dynamically
    assign inlines to specific tabs in the admin interface based on their class names.

    Usage:
        1. Define inline classes inheriting from `TabInline`.
        2. Specify these inline classes in the `inlines` attribute of your `ModelAdmin`.
        3. Use the `fieldsets` attribute to organize fields into tabs, using the `classes`
           attribute to specify which inline classes should appear in each tab.
    Example:
        class CategoryInline(TabInline):
            ...
        class MyModelAdmin(ModelAdmin):
            ...
            fieldsets = (
                ...,
                (
                    "Category Section",
                    {
                        "classes": ["tab", "CategoryInline" ],
                        "fields": [],
                    },
                ),
    """

    @property
    def class_name(self):
        return self.__class__.__name__

    def get_form_queryset(self, obj):
        """
        Gets all nonrelated objects needed for inlines. Method must be implemented.
        """
        return self.model.objects.all()

    def save_new_instance(self, parent, instance):
        """
        Extra save method which can for example update inline instances based on current
        main model object. Method must be implemented.
        """
        pass

    formfield_overrides = {
        models.TextField: {"widget": TinyMCE()},
        models.ImageField: {"widget": ImageUploaderWidget},
        models.FileField: {"widget": ImageUploaderWidget},
    }



from django import forms
from django.core.files.base import ContentFile
from storages.backends.s3 import S3Storage
from webpfield.image_formats import formats_to_convert
from webpfield.settings import DELETE_ORIGINAL, ENABLE_SVG, IMAGE_FIELD_CLASS
from webpfield.utils import convert_to_webp

class CustomWebPStorage(S3Storage):
    def save(self, name, content, max_length=None):
        # get Image extension
        *_, extension = name.split(".")
        # make the extension upper case
        extension = extension.upper()

        # In case of the image is already webP nothing extra need to be done
        if extension not in formats_to_convert:
            return super().save(name, content, max_length=max_length)

        image_bytes = convert_to_webp(content)
        webp_content = ContentFile(image_bytes)
        webp_name = f"{name.split('.')[0]}.webp"
        if not DELETE_ORIGINAL:
            original_image_name = super().save(name, content, max_length=max_length)
        else:
            return super().save(webp_name, webp_content, max_length=max_length)

        webp_name = f"{original_image_name.split('.')[0]}.webp"
        return super().save(webp_name, webp_content, max_length=max_length)


class WebPField(IMAGE_FIELD_CLASS):
    def __init__(self, *args, **kwargs):
        kwargs.update({"storage": CustomWebPStorage()})
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        if ENABLE_SVG:
            kwargs.update({"form_class": forms.FileField})
        return super().formfield(**kwargs)




from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from kavenegar import KavenegarAPI, APIException, HTTPException

from .loggers import logger

User = get_user_model()
kavehnegar_api = KavenegarAPI(settings.KAVENEGAR_API_KEY)


def activate_sms_verified(user):
    user.is_sms_verified = True
    user.save()


def send_validation_sms(phone_number, token):
    params = {
        "receptor": f"{phone_number}",
        "template": "verfiy-ezhoosh",
        "token": f"{str(token)}",
        "type": "sms",
    }
    try:
        kavehnegar_api.verify_lookup(params)
    except APIException as e:
        logger.error(e)
    except HTTPException as e:
        logger.error(e)


def send_notif_sms(phone_number, message):
    params = {
        "receptor": f"{phone_number}",
        "message": f"{message}",
    }
    response = kavehnegar_api.sms_send(params)
    logger.info(response)


def set_user_sms_token(phone, token, timeout=120):
    cache.set(phone, token, timeout=timeout)


def get_user_sms_token(phone):
    try:
        return cache.get(phone)
    except Exception:
        return None



from rest_framework.viewsets import GenericViewSet

from .mixins import (
    CustomListModelMixin,
    CustomCreateModelMixin,
    CustomUpdateModelMixin,
    CustomDestroyModelMixin,
    CustomRetrieveModelMixin,
)


class CustomReadOnlyModelViewSet(
    CustomRetrieveModelMixin, CustomListModelMixin, GenericViewSet
):
    """
    A viewset that provides default `list()` and `retrieve()` actions.
    """

    pass


class CustomModelViewSet(
    CustomCreateModelMixin,
    CustomRetrieveModelMixin,
    CustomUpdateModelMixin,
    CustomDestroyModelMixin,
    CustomListModelMixin,
    GenericViewSet,
):
    """
    A viewset that provides default `create()`, `retrieve()`, `update()`,
    `partial_update()`, `destroy()` and `list()` actions.
    """

    pass



from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.db.models import Manager
from django_lifecycle import LifecycleModelMixin


class ActiveManager(Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True)

    def delete(self):
        return super().get_queryset().update(is_active=False, deleted_at=timezone.now())

    def hard_delete(self):
        return super().get_queryset().delete()

    def restore(self):
        return super().get_queryset().update(is_active=True, deleted_at=None)


class BaseModel(LifecycleModelMixin, models.Model):
    created_at = models.DateTimeField(verbose_name=_("Created At"), auto_now_add=True)
    updated_at = models.DateTimeField(verbose_name=_("Updated At"), auto_now=True)
    is_active = models.BooleanField(verbose_name=_("Is Active"), default=True)
    deleted_at = models.DateTimeField(
        verbose_name=_("Deleted At"), null=True, blank=True
    )
    objects = models.Manager()
    active_objects = ActiveManager()

    class Meta:
        abstract = True



#!/usr/bin/env python
import codecs

from setuptools import find_packages, setup


def read_me(filename):
    return codecs.open(filename, encoding='utf-8').read()


setup(
    name='ez-django-common',
    version='1.0.9',
    python_requires='>=3',
    packages=find_packages(),
    include_package_data=True,
    description=(
        'EZDjangoCommon'
    ),
    url='https://github.com/ezhoosh/EZDjangoCommon',
    download_url='https://pypi.python.org/pypi/ez-django-common/',
    author='ezhoosh',
    author_email='ezhoosh@ezhoosh.com',
    keywords="common",
    license='MIT',
    platforms=['any'],
    install_requires=[
        "django",
        "django-unfold",
        "django-tinymce",
        "django-lifecycle",
        "djangorestframework",
        "loguru",
        "kavenegar",
        "drf-spectacular",
        "django-media-uploader-widget",
        "django-storages",
        "boto3",
        "django-webpfield",

    ],
    long_description=read_me('README.md'),
    long_description_content_type='text/markdown',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
    ],
)


from django.utils.html import format_html
from rest_framework_simplejwt.tokens import RefreshToken


def get_tokens_for_user(user):
    refresh = RefreshToken.for_user(user)

    return {
        "refresh": str(refresh),
        "access": str(refresh.access_token),
    }


def image_preview(instance, field_name: str, width=110, height=110):
    """
    Global utility function to display an image preview in the Django admin panel.

    :param instance: The instance of the model containing the image field.
    :param field_name: The name of the image field (e.g., 'media', 'thumbnail').
    :param width: The width of the image preview (default: 100).
    :param height: The height of the image preview (default: 100).
    :return: HTML formatted string for image preview or a message if no image is available.
    """
    image_field = getattr(instance, field_name, None)
    if image_field and hasattr(image_field, "url"):
        return format_html(
            f'<img src="{image_field.url}" width="{width}" height="{height}" />'
        )
    return "No Image"
